@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2019 Xorg
+*    Copyright © 2014-2020 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -23,19 +23,30 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <libintl.h>
 #include "cpu-x.h"
+#include "ipc.h"
 #include "gui_gtk.h"
 #include "gui_gtk_id.h"
 
-#if PORTABLE_BINARY
-# include "gtk_resources.h"
-#endif
-
 
 /************************* Public function *************************/
 
+/* Load and apply settings from GSettings */
+void load_settings()
+{
+	settings            = g_settings_new(PRGNAME_LOW);
+	theme               = g_settings_get_enum(settings, "gui-theme");
+	opts->refr_time     = g_settings_get_uint(settings, "refresh-time");
+	opts->selected_page = g_settings_get_enum(settings, "default-tab");
+	opts->selected_core = g_settings_get_uint(settings, "default-cpu-core");
+	opts->bw_test       = g_settings_get_uint(settings, "default-cache-test");
+	opts->with_daemon   = g_settings_get_boolean(settings, "always-start-daemon");
+	g_settings_delay(settings);
+}
+
 /* Start CPU-X in GTK mode */
 void start_gui_gtk(int *argc, char **argv[], Labels *data)
 {
@@ -43,27 +54,21 @@ void start_gui_gtk(int *argc, char **argv[], Labels *data)
 	GtkLabels  glab;
 	GThrd      refr     = { &glab, data };
 	GtkBuilder *builder = gtk_builder_new();
-	gchar *prgname      = g_ascii_strdown(PRGNAME, -1);
 	const gchar *ui_files[] = { "cpu-x-gtk-3.12.ui", NULL };
 
-	MSG_VERBOSE(_("Starting GTK GUI..."));
+	MSG_VERBOSE("%s", _("Starting GTK GUI…"));
 	gtk_init(argc, argv);
-	g_set_prgname(prgname);
-	g_free(prgname);
+	g_set_prgname(PRGNAME_LOW);
 
 	/* Build UI from Glade file */
-#if PORTABLE_BINARY
-	g_resources_register(cpu_x_get_resource());
-	for(i = 0; (ui_files[i] != NULL) && (!gtk_builder_add_from_resource(builder, GRESOURCE_UI(ui_files[i]), NULL)); i++);
-#else
 	for(i = 0; (ui_files[i] != NULL) && (!gtk_builder_add_from_file(builder, data_path(ui_files[i]), NULL)); i++);
-#endif /* PORTABLE_BINARY */
 	if(ui_files[i] == NULL)
 	{
-		MSG_ERROR(_("failed to import UI in GtkBuilder"));
+		MSG_ERROR("%s", _("failed to import UI in GtkBuilder"));
 		exit(EXIT_FAILURE);
 	}
 
+	/* Set widgets */
 	get_widgets(builder, &glab);
 	g_object_unref(G_OBJECT(builder));
 	set_colors (&glab);
@@ -72,80 +77,24 @@ void start_gui_gtk(int *argc, char **argv[], Labels *data)
 	set_signals(&glab, data, &refr);
 	labels_free(data);
 
-	if(getuid())
-		warning_window(glab.mainwindow);
+	/* Bind settings to get_widgets */
+	g_settings_bind(settings, "refresh-time",        glab.refreshtime,      "value",     G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind(settings, "gui-theme",           glab.theme,            "active-id", G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind(settings, "default-tab",         glab.defaulttab,       "active-id", G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind(settings, "default-cpu-core",    glab.defaultcore,      "active",    G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind(settings, "default-cache-test",  glab.defaultcachetest, "active",    G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind(settings, "always-start-daemon", glab.startdaemon,      "active",    G_SETTINGS_BIND_DEFAULT);
 
-#ifdef __x86_64__
-	if(PORTABLE_BINARY && (new_version[0] != NULL) && !opts->update)
-		new_version_window(glab.mainwindow);
-#endif /* __x86_64__ */
-
-	g_timeout_add_seconds(opts->refr_time, (gpointer)grefresh, &refr);
+	g_timeout_add_seconds_full(G_PRIORITY_DEFAULT, opts->refr_time, (gpointer)grefresh, &refr, modify_refresh_time);
 	gtk_main();
+
+	if(data->reload)
+		execvp((*argv)[0], *argv);
 }
 
 
 /************************* Private functions *************************/
 
-/* Print a window which allows to restart CPU-X as root */
-static void warning_window(GtkWidget *mainwindow)
-{
-	char *const cmd[] = { "cpu-x_polkit", NULL };
-	GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(mainwindow),
-		GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_MESSAGE_WARNING,
-		GTK_BUTTONS_NONE,
-		_("Root privileges are required to work properly"));
-
-	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-		_("Some informations will not be retrievable"));
-
-	gtk_dialog_add_button(GTK_DIALOG(dialog), _("Ignore"), GTK_RESPONSE_REJECT);
-
-#if !(PORTABLE_BINARY)
-	if(!getenv("APPIMAGE") && command_exists("pkexec") && command_exists("cpu-x_polkit"))
-		gtk_dialog_add_button(GTK_DIALOG(dialog), _("Run as root"), GTK_RESPONSE_ACCEPT);
-#endif /* !(PORTABLE_BINARY) */
-
-	if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-		execvp(cmd[0], cmd);
-
-	gtk_widget_destroy(dialog);
-}
-
-/* In portable version, inform when a new version is available and ask for update */
-static void new_version_window(GtkWidget *mainwindow)
-{
-	GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(mainwindow),
-		GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_MESSAGE_INFO,
-		GTK_BUTTONS_NONE,
-		_("A new version of %s is available!"), PRGNAME);
-#if 0
-	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-		_("Do you want to update %s to version %s after exit?\n"
-		"It will erase this binary file (%s) by the new version."),
-		PRGNAME, new_version[0], binary_name);
-	gtk_dialog_add_buttons(GTK_DIALOG(dialog), _("Not now"), GTK_RESPONSE_REJECT, _("Update"), GTK_RESPONSE_ACCEPT, NULL);
-
-	if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-		opts->update = true;
-#else
-	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-		_("Version %s is available. Unfortunately, this portable format is now deprecated in favor of AppImage.\n"
-		"You can download %s AppImage on the official download page.\n"),
-		new_version[0], PRGNAME);
-	GtkWidget *link = gtk_link_button_new_with_label(DOWNLOAD, _("Download page"));
-	gtk_widget_show(link);
-	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), GTK_WIDGET(link), GTK_RESPONSE_OK);
-	gtk_dialog_add_button(GTK_DIALOG(dialog), _("Ok"), GTK_RESPONSE_CLOSE);
-	gtk_dialog_run(GTK_DIALOG(dialog));
-	gtk_widget_destroy(link);
-#endif
-
-	gtk_widget_destroy(dialog);
-}
-
 /* Refresh dynamic values */
 static gboolean grefresh(GThrd *refr)
 {
@@ -175,12 +124,12 @@ static gboolean grefresh(GThrd *refr)
 				gtk_label_set_text(GTK_LABEL(glab->gtktab_system[VALUE][i]), data->tab_system[VALUE][i]);
 			break;
 		case NO_GRAPHICS:
-			for(i = 0; i < data->gpu_count; i += GPUFIELDS)
+			for(i = 0; i < data->gpu_count; i++)
 			{
-				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1TEMPERATURE + i]), data->tab_graphics[VALUE][GPU1TEMPERATURE + i]);
-				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1USAGE       + i]), data->tab_graphics[VALUE][GPU1USAGE       + i]);
-				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1CORECLOCK   + i]), data->tab_graphics[VALUE][GPU1CORECLOCK   + i]);
-				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1MEMCLOCK    + i]), data->tab_graphics[VALUE][GPU1MEMCLOCK    + i]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS]);
 			}
 			break;
 		case NO_BENCH:
@@ -192,7 +141,78 @@ static gboolean grefresh(GThrd *refr)
 			break;
 	}
 
-	return G_SOURCE_CONTINUE;
+	/* Destroy current timeout when refresh time is updated
+	Note: modify_refresh_time() will create a new timeout */
+	return (opts->refr_time == g_settings_get_uint(settings, "refresh-time"));
+}
+
+/* Create new timeout when old one is destroyed */
+static void modify_refresh_time(gpointer data)
+{
+	opts->refr_time = g_settings_get_uint(settings, "refresh-time");
+	g_timeout_add_seconds_full(G_PRIORITY_DEFAULT, opts->refr_time, (gpointer)grefresh, data, modify_refresh_time);
+}
+
+/* Show settings window */
+static void open_settings_window(GtkWidget *__button, GtkLabels *glab)
+{
+	UNUSED(__button);
+	gtk_widget_show(GTK_WIDGET(glab->settingswindow));
+}
+
+/* Hide settings window and revert changes */
+static gboolean hide_settings_window(GtkWidget *widget, GdkEvent *event, GtkLabels *glab)
+{
+	UNUSED(event);
+	UNUSED(glab);
+	g_settings_revert(settings);
+	gtk_widget_hide(widget);
+	return TRUE;
+}
+
+/* Hide settings window and revert changes */
+static void close_settings_window(GtkWidget *__button, GtkLabels *glab)
+{
+	UNUSED(__button);
+	g_settings_revert(settings);
+	gtk_widget_hide(GTK_WIDGET(glab->settingswindow));
+}
+
+/* Hide settings window and apply changes */
+static void save_settings(GtkWidget *__button, GtkLabels *glab)
+{
+	UNUSED(__button);
+	theme = g_settings_get_enum(settings, "gui-theme");
+	set_colors(glab);
+	g_settings_apply(settings);
+	gtk_widget_hide(GTK_WIDGET(glab->settingswindow));
+}
+
+/* Start daemon and reload CPU-X */
+static void reload_with_daemon(GtkWidget *button, GThrd *refr)
+{
+	Labels    *(data) = refr->data;
+	GtkLabels *(glab) = refr->glab;
+
+	gtk_widget_set_sensitive(button, false);
+	const char *msg = start_daemon(true);
+
+	if(msg == NULL)
+	{
+		data->reload = true;
+		gtk_main_quit();
+	}
+	else
+	{
+		GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(glab->mainwindow),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_MESSAGE_WARNING,
+			GTK_BUTTONS_CLOSE,
+			"%s", _(msg));
+		if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_CLOSE)
+			gtk_widget_destroy(GTK_WIDGET(dialog));
+		gtk_widget_set_sensitive(button, true);
+	}
 }
 
 /* Event in CPU tab when Core number is changed */
@@ -214,8 +234,9 @@ static void change_activetest(GtkComboBox *box, Labels *data)
 }
 
 /* Events in Bench tab when a benchmark start/stop */
-static void start_benchmark_bg(GtkSwitch *gswitch, GdkEvent *event, GThrd *refr)
+static void start_benchmark_bg(GtkSwitch *gswitch, GdkEvent *__event, GThrd *refr)
 {
+	UNUSED(__event);
 	Labels *data = refr->data;
 
 	if(!data->b_data->run)
@@ -252,10 +273,9 @@ static void change_benchsensitive(GtkLabels *glab, Labels *data)
 	if(data->b_data->run)
 	{
 		skip = false;
-#if GTK_CHECK_VERSION(3, 15, 0) || PORTABLE_BINARY
-		if(gtk_check_version(3, 15, 0) == NULL)
-			gtk_switch_set_state(GTK_SWITCH(glab->gtktab_bench[VALUE][indA]), true);
-#endif /* GTK_CHECK_VERSION(3, 15, 0) || PORTABLE_BINARY */
+#if GTK_CHECK_VERSION(3, 15, 0)
+		gtk_switch_set_state(GTK_SWITCH(glab->gtktab_bench[VALUE][indA]), true);
+#endif /* GTK_CHECK_VERSION(3, 15, 0) */
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(glab->gtktab_bench[VALUE][indP]),
 			(double) data->b_data->elapsed / (data->b_data->duration * 60));
 		gtk_widget_set_sensitive(glab->gtktab_bench[VALUE][indS],         false);
@@ -264,10 +284,9 @@ static void change_benchsensitive(GtkLabels *glab, Labels *data)
 	else if(!data->b_data->run && !skip)
 	{
 		skip = true;
-#if GTK_CHECK_VERSION(3, 15, 0) || PORTABLE_BINARY
-		if(gtk_check_version(3, 15, 0) == NULL)
-			gtk_switch_set_state(GTK_SWITCH(glab->gtktab_bench[VALUE][indA]),  false);
-#endif /* GTK_CHECK_VERSION(3, 15, 0) || PORTABLE_BINARY */
+#if GTK_CHECK_VERSION(3, 15, 0)
+		gtk_switch_set_state(GTK_SWITCH(glab->gtktab_bench[VALUE][indA]),  false);
+#endif /* GTK_CHECK_VERSION(3, 15, 0) */
 		gtk_switch_set_active(GTK_SWITCH(glab->gtktab_bench[VALUE][indA]), false);
 		gtk_widget_set_sensitive(glab->gtktab_bench[VALUE][indS],          true);
 		gtk_widget_set_sensitive(glab->gtktab_bench[VALUE][PARAMTHREADS],  true);
@@ -312,16 +331,28 @@ static void get_widgets(GtkBuilder *builder, GtkLabels *glab)
 {
 	int i;
 
-	glab->mainwindow  = GTK_WIDGET(gtk_builder_get_object(builder, "mainwindow"));
-	glab->closebutton = GTK_WIDGET(gtk_builder_get_object(builder, "closebutton"));
-	glab->labprgver	  = GTK_WIDGET(gtk_builder_get_object(builder, "labprgver"));
-	glab->footer      = GTK_WIDGET(gtk_builder_get_object(builder, "footer_box"));
-	glab->notebook    = GTK_WIDGET(gtk_builder_get_object(builder, "header_notebook"));
-	glab->logocpu     = GTK_WIDGET(gtk_builder_get_object(builder, "proc_logocpu"));
-	glab->activecore  = GTK_WIDGET(gtk_builder_get_object(builder, "trg_activecore"));
-	glab->activetest  = GTK_WIDGET(gtk_builder_get_object(builder, "test_activetest"));
-	glab->logoprg     = GTK_WIDGET(gtk_builder_get_object(builder, "about_logoprg"));
-	glab->butcol      = GTK_WIDGET(gtk_builder_get_object(builder, "colorbutton"));
+	glab->mainwindow     = GTK_WIDGET(gtk_builder_get_object(builder, "mainwindow"));
+	glab->settingsbutton = GTK_WIDGET(gtk_builder_get_object(builder, "settingsbutton"));
+	glab->daemonbutton   = GTK_WIDGET(gtk_builder_get_object(builder, "daemonbutton"));
+	glab->labprgver	     = GTK_WIDGET(gtk_builder_get_object(builder, "labprgver"));
+	glab->footer         = GTK_WIDGET(gtk_builder_get_object(builder, "footer_box"));
+	glab->notebook       = GTK_WIDGET(gtk_builder_get_object(builder, "header_notebook"));
+	glab->logocpu        = GTK_WIDGET(gtk_builder_get_object(builder, "proc_logocpu"));
+	glab->activecore     = GTK_WIDGET(gtk_builder_get_object(builder, "trg_activecore"));
+	glab->activetest     = GTK_WIDGET(gtk_builder_get_object(builder, "test_activetest"));
+	glab->logoprg        = GTK_WIDGET(gtk_builder_get_object(builder, "about_logoprg"));
+	glab->butcol         = GTK_WIDGET(gtk_builder_get_object(builder, "colorbutton"));
+
+	glab->settingswindow   = GTK_WIDGET(gtk_builder_get_object(builder, "settingswindow"));
+	glab->validatebutton   = GTK_WIDGET(gtk_builder_get_object(builder, "validatebutton"));
+	glab->cancelbutton     = GTK_WIDGET(gtk_builder_get_object(builder, "cancelbutton"));
+	glab->refreshtime      = GTK_WIDGET(gtk_builder_get_object(builder, "refreshtime_val"));
+	glab->theme            = GTK_WIDGET(gtk_builder_get_object(builder, "theme_val"));
+	glab->defaulttab       = GTK_WIDGET(gtk_builder_get_object(builder, "defaulttab_val"));
+	glab->defaultcore      = GTK_WIDGET(gtk_builder_get_object(builder, "defaultcore_val"));
+	glab->defaultcachetest = GTK_WIDGET(gtk_builder_get_object(builder, "defaultcachetest_val"));
+	glab->startdaemon      = GTK_WIDGET(gtk_builder_get_object(builder, "startdaemon"));
+
 	gtk_widget_set_name(glab->footer, "footer_box");
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(glab->notebook), opts->selected_page);
 
@@ -337,7 +368,6 @@ static void get_widgets(GtkBuilder *builder, GtkLabels *glab)
 		gtk_widget_set_name(glab->gtktab_cpu[VALUE][i], "value");
 	}
 
-
 	/* Tab Caches */
 	for(i = L1SIZE; i < LASTCACHES; i++)
 	{
@@ -400,35 +430,15 @@ static void get_widgets(GtkBuilder *builder, GtkLabels *glab)
 		glab->gtktab_about[i] = GTK_WIDGET(gtk_builder_get_object(builder, objectabout[i]));
 }
 
-#if PORTABLE_BINARY
-/* Backport gtk_css_provider_load_from_resource() for GTK < 3.16
-https://github.com/GNOME/gtk/blob/master/gtk/gtkcssprovider.c#L1943 */
-void __builtin_gtk_css_provider_load_from_resource(GtkCssProvider *css_provider, const gchar *resource_path)
-{
-	GFile *file;
-	gchar *uri, *escaped;
-
-	g_return_if_fail (GTK_IS_CSS_PROVIDER (css_provider));
-	g_return_if_fail (resource_path != NULL);
-
-	escaped = g_uri_escape_string (resource_path, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
-	uri = g_strconcat ("resource://", escaped, NULL);
-	g_free (escaped);
-
-	file = g_file_new_for_uri (uri);
-	g_free (uri);
-
-	gtk_css_provider_load_from_file (css_provider, file, NULL);
-	g_object_unref (file);
-}
-#endif /* PORTABLE_BINARY */
-
 static gboolean is_dark_theme(GtkLabels *glab)
 {
 	gdouble contrast;
 	GdkRGBA *fg, *bg;
 	GtkStyleContext *context;
 
+	if (theme != AUTO)
+		return (theme == DARK);
+
 	context = gtk_widget_get_style_context(GTK_WIDGET(glab->mainwindow));
 	gtk_style_context_get(context, GTK_STATE_FLAG_NORMAL,
 		GTK_STYLE_PROPERTY_COLOR, &fg,
@@ -453,15 +463,7 @@ static void set_colors(GtkLabels *glab)
 		filename = is_dark_theme(glab) ? g_strdup("cpu-x-gtk-3.12-dark.css") : g_strdup("cpu-x-gtk-3.12.css");
 
 	gtk_style_context_add_provider_for_screen(gdk_screen_get_default(), GTK_STYLE_PROVIDER(provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-
-#if PORTABLE_BINARY
-	if(gtk_check_version(3, 15, 0) == NULL) // GTK 3.16 or newer
-		gtk_css_provider_load_from_resource(provider, GRESOURCE_CSS(filename));
-	else // GTK 3.12 to 3.14
-		__builtin_gtk_css_provider_load_from_resource(provider, GRESOURCE_CSS(filename));
-#else
 	gtk_css_provider_load_from_path(provider, data_path(filename), NULL);
-#endif /* PORTABLE_BINARY */
 
 	g_free(filename);
 	g_object_unref(provider);
@@ -478,18 +480,9 @@ static void set_logos(GtkLabels *glab, Labels *data)
 	         gtk_widget_get_allocated_width(glab->gtktab_cpu[VALUE][VENDOR]) - 6;
 	height = (gtk_widget_get_allocated_height(glab->gtktab_cpu[VALUE][VENDOR]) + 4) * 4;
 
-	if(PORTABLE_BINARY)
-	{
-		cpu_pixbuf     = gdk_pixbuf_new_from_resource_at_scale(format(GRESOURCE_LOGOS("%s.png"), data->tab_cpu[VALUE][VENDOR]), width, height, TRUE, &error);
-		unknown_pixbuf = gdk_pixbuf_new_from_resource_at_scale(GRESOURCE_LOGOS("Unknown.png"), width, height, TRUE, NULL);
-		prg_pixbuf     = gdk_pixbuf_new_from_resource_at_scale(GRESOURCE_LOGOS("CPU-X.png"), prg_size, prg_size, TRUE, NULL);
-	}
-	else
-	{
-		cpu_pixbuf     = gdk_pixbuf_new_from_file_at_scale(data_path(format("%s.png", data->tab_cpu[VALUE][VENDOR])), width, height, TRUE, &error);
-		unknown_pixbuf = gdk_pixbuf_new_from_file_at_scale(data_path("Unknown.png"), width, height, TRUE, NULL);
-		prg_pixbuf     = gdk_pixbuf_new_from_file_at_scale(data_path("CPU-X.png"), prg_size, prg_size, TRUE, NULL);
-	}
+	cpu_pixbuf     = gdk_pixbuf_new_from_file_at_scale(data_path(format("%s.png", data->tab_cpu[VALUE][VENDOR])), width, height, TRUE, &error);
+	unknown_pixbuf = gdk_pixbuf_new_from_file_at_scale(data_path("Unknown.png"), width, height, TRUE, NULL);
+	prg_pixbuf     = gdk_pixbuf_new_from_file_at_scale(data_path("CPU-X.png"), prg_size, prg_size, TRUE, NULL);
 
 	gtk_window_set_icon(GTK_WINDOW(glab->mainwindow),   prg_pixbuf);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(glab->logocpu), cpu_pixbuf);
@@ -506,16 +499,28 @@ static void set_logos(GtkLabels *glab, Labels *data)
 static void set_labels(GtkLabels *glab, Labels *data)
 {
 	int i;
+	const int pkcheck = system(format("pkcheck --action-id org.freedesktop.policykit.exec --process %u > /dev/null 2>&1", getpid()));
 	const gint width1 = gtk_widget_get_allocated_width(glab->gtktab_system[VALUE][COMPILER]);
 	const gint width2 = width1 - gtk_widget_get_allocated_width(glab->gtktab_system[VALUE][USED]) - 6;
 	GtkRequisition requisition;
 
 	/* Footer label */
 	gtk_label_set_text(GTK_LABEL(glab->labprgver), data->tab_about[VERSIONSTR]);
+	gtk_widget_set_sensitive(glab->daemonbutton, false);
+	if(DAEMON_UP)
+		gtk_widget_set_tooltip_text(glab->daemonbutton, _("Connected to daemon"));
+	else if(WEXITSTATUS(pkcheck) > 2)
+		gtk_widget_set_tooltip_text(glab->daemonbutton, _("No polkit authentication agent found"));
+	else
+	{
+		gtk_widget_set_sensitive(glab->daemonbutton, true);
+		gtk_widget_set_tooltip_text(glab->daemonbutton, _("Ask password to start daemon in background"));
+	}
 
 	/* Various labels to translate */
 	for(i = TABCPU; i < LASTOBJ; i++)
 		gtk_label_set_text(GTK_LABEL(glab->gtktrad[i]), data->objects[i]);
+	gtk_button_set_label(GTK_BUTTON(glab->daemonbutton), _("Start daemon"));
 
 	/* Tab CPU */
 	for(i = VENDOR; i < LASTCPU; i++)
@@ -591,7 +596,6 @@ static void set_labels(GtkLabels *glab, Labels *data)
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(glab->gtktab_bench[VALUE][i]), data->tab_bench[VALUE][i]);
 		gtk_widget_set_size_request(glab->gtktab_bench[VALUE][i], width1, -1);
 	}
-
 	gtk_spin_button_set_increments(GTK_SPIN_BUTTON(glab->gtktab_bench[VALUE][PARAMDURATION]), 1, 60);
 	gtk_spin_button_set_increments(GTK_SPIN_BUTTON(glab->gtktab_bench[VALUE][PARAMTHREADS]),  1, 1);
 	gtk_spin_button_set_range     (GTK_SPIN_BUTTON(glab->gtktab_bench[VALUE][PARAMDURATION]), 1, 60 * 24);
@@ -600,6 +604,20 @@ static void set_labels(GtkLabels *glab, Labels *data)
 	/* Tab About */
 	for(i = DESCRIPTION; i < LASTABOUT; i++)
 		gtk_label_set_text(GTK_LABEL(glab->gtktab_about[i]), data->tab_about[i]);
+
+	/* Settings window */
+	gtk_spin_button_set_range     (GTK_SPIN_BUTTON(glab->refreshtime),  1, G_MAXUSHORT);
+	gtk_spin_button_set_increments(GTK_SPIN_BUTTON(glab->refreshtime),  1, 60);
+	for (i = NO_CPU; i <= NO_ABOUT; i++)
+		gtk_combo_box_text_insert(GTK_COMBO_BOX_TEXT(glab->defaulttab), i, nicktab[i], format(_("%s"), data->objects[i]));
+	gtk_combo_box_set_active(GTK_COMBO_BOX(glab->defaulttab), opts->selected_page);
+	for(i = 0; i < data->cpu_count; i++)
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(glab->defaultcore), format(_("Core #%i"), i));
+	gtk_combo_box_set_active(GTK_COMBO_BOX(glab->defaultcore), opts->selected_core);
+	for(i = 0; i < data->w_data->test_count; i++)
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(glab->defaultcachetest), data->w_data->test_name[i]);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(glab->defaultcachetest), opts->bw_test);
+	gtk_widget_set_sensitive(GTK_WIDGET(glab->defaultcachetest), data->cache_count > 0);
 }
 
 /* Call defined functions on signals */
@@ -607,18 +625,26 @@ static void set_signals(GtkLabels *glab, Labels *data, GThrd *refr)
 {
 	int i;
 
-	g_signal_connect(glab->mainwindow,  "destroy", G_CALLBACK(gtk_main_quit),     NULL);
-	g_signal_connect(glab->closebutton, "clicked", G_CALLBACK(gtk_main_quit),     NULL);
-	g_signal_connect(glab->activecore,  "changed", G_CALLBACK(change_activecore), data);
-	g_signal_connect(glab->activetest,  "changed", G_CALLBACK(change_activetest), data);
+	g_signal_connect(glab->mainwindow,     "destroy",      G_CALLBACK(gtk_main_quit),        NULL);
+	g_signal_connect(glab->settingswindow, "delete-event", G_CALLBACK(hide_settings_window), NULL);
+	g_signal_connect(glab->settingsbutton, "clicked",      G_CALLBACK(open_settings_window), glab);
+	g_signal_connect(glab->daemonbutton,   "clicked",      G_CALLBACK(reload_with_daemon),   refr);
+	g_signal_connect(glab->activecore,     "changed",      G_CALLBACK(change_activecore),    data);
+	g_signal_connect(glab->activetest,     "changed",      G_CALLBACK(change_activetest),    data);
 
+	/* Tab Bench */
 	g_signal_connect(glab->gtktab_bench[VALUE][PRIMESLOWRUN],  "button-press-event", G_CALLBACK(start_benchmark_bg), refr);
 	g_signal_connect(glab->gtktab_bench[VALUE][PRIMEFASTRUN],  "button-press-event", G_CALLBACK(start_benchmark_bg), refr);
 	g_signal_connect(glab->gtktab_bench[VALUE][PARAMDURATION], "value-changed",      G_CALLBACK(change_benchparam),  data);
 	g_signal_connect(glab->gtktab_bench[VALUE][PARAMTHREADS],  "value-changed",      G_CALLBACK(change_benchparam),  data);
 
+	/* Tab System */
 	for(i = BARUSED; i < LASTBAR; i++)
 		g_signal_connect(G_OBJECT(glab->bar[i]),  "draw", G_CALLBACK(fill_frame), refr);
+
+	/* Settings window */
+	g_signal_connect(glab->validatebutton, "clicked", G_CALLBACK(save_settings), glab);
+	g_signal_connect(glab->cancelbutton,   "clicked", G_CALLBACK(close_settings_window), glab);
 }
 
 /* Draw bars in Memory tab */
