@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2019 Xorg
+*    Copyright © 2014-2020 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -24,25 +24,32 @@
 #ifndef _GUI_GTK_H_
 #define _GUI_GTK_H_
 
-
 #include <gtk/gtk.h>
 #include <cairo.h>
 #include <glib.h>
 
-#define GRESOURCE_UI(file)    g_strconcat("/cpu-x/ui/",    file, NULL)
-#define GRESOURCE_CSS(file)   g_strconcat("/cpu-x/css/",   file, NULL)
-#define GRESOURCE_LOGOS(file) g_strconcat("/cpu-x/logos/", file, NULL)
-
 typedef struct
 {
 	/* Common */
 	GtkWidget *mainwindow;
+	GtkWidget *settingsbutton;
 	GtkWidget *notebook;
 	GtkWidget *footer;
 	GtkWidget *labprgver;
-	GtkWidget *closebutton;
+	GtkWidget *daemonbutton;
 	GtkWidget *gtktrad[LASTOBJ];
 
+	/* Settings */
+	GtkWidget *settingswindow;
+	GtkWidget *validatebutton;
+	GtkWidget *cancelbutton;
+	GtkWidget *refreshtime;
+	GtkWidget *theme;
+	GtkWidget *defaulttab;
+	GtkWidget *defaultcore;
+	GtkWidget *defaultcachetest;
+	GtkWidget *startdaemon;
+
 	/* Tab CPU */
 	GtkWidget *logocpu;
 	GtkWidget *gtktab_cpu[2][LASTCPU];
@@ -79,27 +86,47 @@ typedef struct
 
 } GtkLabels; /* Useful GtkWidgets */
 
+static enum {
+	AUTO,
+	LIGHT,
+	DARK
+} theme = AUTO;
+
 typedef struct
 {
 	GtkLabels *glab;
 	Labels *data;
 } GThrd; /* Used to refresh GUI */
 
-
 /********************************** GUI  **********************************/
 
+/* Load and apply settings from GSettings */
+void load_settings(void);
+
 /* Start CPU-X in GTK mode */
 void start_gui_gtk(int *argc, char **argv[], Labels *data);
 
-/* Print a window which allows to restart CPU-X as root */
-static void warning_window(GtkWidget *mainwindow);
-
-/* In portable version, inform when a new version is available and ask for update */
-static void new_version_window(GtkWidget *mainwindow);
-
 /* Refresh dynamic values */
 static gboolean grefresh(GThrd *refr);
 
+/* Create new timeout when old one is destroyed */
+static void modify_refresh_time(gpointer data);
+
+/* Show settings window */
+static void open_settings_window(GtkWidget *button, GtkLabels *glab);
+
+/* Hide settings window and revert changes */
+static gboolean hide_settings_window(GtkWidget *widget, GdkEvent *event, GtkLabels *glab);
+
+/* Hide settings window and revert changes */
+static void close_settings_window(GtkWidget *button, GtkLabels *glab);
+
+/* Hide settings window and apply changes */
+static void save_settings(GtkWidget *button, GtkLabels *glab);
+
+/* Start daemon and reload CPU-X */
+static void reload_with_daemon(GtkWidget *button, GThrd *refr);
+
 /* Event in CPU tab when Core number is changed */
 static void change_activecore(GtkComboBox *box, Labels *data);
 
@@ -139,5 +166,4 @@ static void set_signals(GtkLabels *glab, Labels *data, GThrd *refr);
 /* Draw bars in Memory tab */
 void fill_frame(GtkWidget *widget, cairo_t *cr, GThrd *refr);
 
-
 #endif /* _GUI_GTK_H_ */
