@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2019 Xorg
+*    Copyright © 2014-2020 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,9 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <ncurses.h>
+#include <term.h>
 #include <math.h>
 #include <libintl.h>
 #include "cpu-x.h"
@@ -62,15 +64,30 @@ static const Colors color[] =
 /* Start CPU-X in NCurses mode */
 void start_tui_ncurses(Labels *data)
 {
-	int startx, starty, ch = 0;
+	int err = 0, ch = 0;
 	const SizeInfo info = { .height = LINE_COUNT, .width = 70, .start = 1, .tb = 2, .tm = 26, .te = 48 };
 	NThrd refr = { .data = data, .info = info };
 	WINDOW *win;
 
-	MSG_VERBOSE(_("Starting NCurses TUI..."));
+	MSG_VERBOSE("%s", _("Starting NCurses TUI…"));
 	if(!getenv("TERMINFO"))
 		setenv("TERMINFO", TERMINFODIR, 0);
 	freopen("/dev/null", "a", stderr);
+	if(setupterm(NULL, STDOUT_FILENO, &err))
+	{
+		if(err == -1)
+		{
+			MSG_ERROR("%s", _("FATAL ERROR: terminfo database could not be found (try to set TERMINFO environment variable)"));
+			exit(255);
+		}
+		else
+		{
+			MSG_WARNING(_("Failed to set up %s terminal (err=%i); falling back to %s"), getenv("TERM"), err, DEFAULT_TERM);
+			setenv("TERM", DEFAULT_TERM, 1);
+			setupterm(DEFAULT_TERM, STDOUT_FILENO, &err);
+		}
+	}
+
 	initscr();
 	cbreak();
 	noecho();
@@ -90,24 +107,14 @@ void start_tui_ncurses(Labels *data)
 			init_pair(pair, color[pair].foreground, color[pair].background);
 	}
 
-#ifdef __x86_64__
-	if(PORTABLE_BINARY && (new_version[0] != NULL) && !opts->update)
-	{
-		print_new_version();
-		erase();
-	}
-#endif /* __x86_64__ */
-
-	starty   = (LINES - info.height) / 2; /* Calculating for a center placement of the window */
-	startx   = (COLS  - info.width)  / 2;
-	win      = newwin(info.height, info.width, starty, startx);
+	win      = newwin(info.height, info.width, 0, 0);
 	refr.win = win;
 
-	refresh();
-	main_win(win, info, data);
-	(*func_ptr[opts->selected_page])(win, info, data);
-	timeout(opts->refr_time * 1000);
-	printw(_("Press 'h' to see help.\n"));
+	if(resize_window(win, info))
+	{
+		main_win(win, info, data);
+		(*func_ptr[opts->selected_page])(win, info, data);
+	}
 
 	while(ch != 'q')
 	{
@@ -119,8 +126,11 @@ void start_tui_ncurses(Labels *data)
 				if(opts->selected_page > NO_CPU)
 				{
 					opts->selected_page--;
-					main_win(win, info, data);
-					(*func_ptr[opts->selected_page])(win, info, data);
+					if(resize_window(win, info))
+					{
+						main_win(win, info, data);
+						(*func_ptr[opts->selected_page])(win, info, data);
+					}
 				}
 				break;
 			case KEY_RIGHT:
@@ -128,8 +138,11 @@ void start_tui_ncurses(Labels *data)
 				if(opts->selected_page < NO_ABOUT)
 				{
 					opts->selected_page++;
-					main_win(win, info, data);
-					(*func_ptr[opts->selected_page])(win, info, data);
+					if(resize_window(win, info))
+					{
+						main_win(win, info, data);
+						(*func_ptr[opts->selected_page])(win, info, data);
+					}
 				}
 				break;
 			case KEY_DOWN:
@@ -201,25 +214,28 @@ void start_tui_ncurses(Labels *data)
 			case 'h':
 				erase();
 				print_help();
-				erase();
-				refresh();
-				main_win(win, info, data);
-				(*func_ptr[opts->selected_page])(win, info, data);
+				if(resize_window(win, info))
+				{
+					main_win(win, info, data);
+					(*func_ptr[opts->selected_page])(win, info, data);
+				}
 				break;
 			case ERR:
 				/* Refresh dynamic labels */
-				if(opts->selected_page == NO_CPU || opts->selected_page == NO_CACHES || opts->selected_page == NO_SYSTEM || opts->selected_page == NO_GRAPHICS || opts->selected_page == NO_BENCH)
+				if(resize_window(win, info))
+				{
 					nrefresh(&refr);
+					main_win(win, info, data);
+					(*func_ptr[opts->selected_page])(win, info, data);
+				}
 				break;
 			case KEY_RESIZE:
 				/* Resize window */
-				erase();
-				starty = (LINES - info.height) / 2;
-				startx = (COLS - info.width) / 2;
-				mvwin(win, starty, startx);
-				refresh();
-				main_win(win, info, data);
-				(*func_ptr[opts->selected_page])(win, info, data);
+				if(resize_window(win, info))
+				{
+					main_win(win, info, data);
+					(*func_ptr[opts->selected_page])(win, info, data);
+				}
 				break;
 			default:
 				break;
@@ -233,6 +249,32 @@ void start_tui_ncurses(Labels *data)
 
 /************************* Private functions *************************/
 
+/* Put window in the center of the screen */
+static bool resize_window(WINDOW *pwin, const SizeInfo info)
+{
+	bool ret = true;
+	const int starty = (LINES - info.height) / 2;
+	const int startx = (COLS  - info.width)  / 2;
+
+	erase();
+	if((startx < 0) || (starty < 0))
+	{
+		printw("%s", _("Window is too small!\n"));
+		timeout(-1);
+		ret = false;
+	}
+	else
+	{
+		wresize(pwin, info.height, info.width);
+		mvwin(pwin, starty, startx);
+		printw("%s", _("Press 'h' to see help.\n"));
+		timeout(opts->refr_time * 1000);
+	}
+	refresh();
+
+	return ret;
+}
+
 /* Clean window */
 static void wclrscr(WINDOW *pwin)
 {
@@ -246,15 +288,6 @@ static void wclrscr(WINDOW *pwin)
 	}
 }
 
-/* Clean line */
-static void wclrline(WINDOW *pwin, enum EnLines line, unsigned start, unsigned end)
-{
-	unsigned x;
-
-	for(x = start; x < end; x++)
-		mvwprintwc(pwin, line, x, DEFAULT_COLOR, " ");
-}
-
 /* Similar to mvwprintw, but specify a color pair */
 static int mvwprintwc(WINDOW *win, int y, int x, enum EnColors pair, const char *fmt, ...)
 {
@@ -281,11 +314,12 @@ static int mvwprintw2c(WINDOW *win, int y, int x, const char *fmt, ...)
 	char *s1, *s2, *f1, *f2, *ptr = strdup(fmt);
 	enum EnColors pair;
 	va_list args;
+	const size_t ptr_len = strlen(ptr) + 1;
 
 	/* Retrive args */
 	va_start(args, fmt);
 	f2 = strstr(fmt, ": ") + 1;
-	f1 = strcat(strtok(ptr, ": "), ":");
+	f1 = strncat(strtok(ptr, ": "), ":", ptr_len);
 	s1 = va_arg(args, char *);
 	s2 = va_arg(args, char *);
 
@@ -319,11 +353,10 @@ static void nrefresh(NThrd *refr)
 	Labels *(data) = refr->data;
 	SizeInfo info  = refr->info;
 
-	do_refresh(data);
-
 	switch(opts->selected_page)
 	{
 		case NO_CPU:
+			do_refresh(data);
 			mvwprintw2c(win, LINE_4,  info.tm, "%11s: %s", data->tab_cpu[NAME][VOLTAGE],     data->tab_cpu[VALUE][VOLTAGE]);
 			mvwprintw2c(win, LINE_6,  info.te, "%9s: %s",  data->tab_cpu[NAME][TEMPERATURE], data->tab_cpu[VALUE][TEMPERATURE]);
 			mvwprintw2c(win, LINE_12, info.tb, "%14s: %s", data->tab_cpu[NAME][CORESPEED],   data->tab_cpu[VALUE][CORESPEED]);
@@ -331,6 +364,7 @@ static void nrefresh(NThrd *refr)
 			mvwprintw2c(win, LINE_15, info.tb, "%14s: %s", data->tab_cpu[NAME][USAGE],       data->tab_cpu[VALUE][USAGE]);
 			break;
 		case NO_CACHES:
+			do_refresh(data);
 			line = LINE_2;
 			for(i = L1SPEED; i < data->cache_count * CACHEFIELDS; i += CACHEFIELDS)
 			{
@@ -339,6 +373,7 @@ static void nrefresh(NThrd *refr)
 			}
 			break;
 		case NO_SYSTEM:
+			do_refresh(data);
 			mvwprintw2c(win, LINE_4,  info.tb, "%13s: %s", data->tab_system[NAME][UPTIME],   data->tab_system[VALUE][UPTIME]);
 			for(i = USED; i < LASTSYSTEM; i++)
 			{
@@ -347,6 +382,7 @@ static void nrefresh(NThrd *refr)
 			}
 			break;
 		case NO_GRAPHICS:
+			do_refresh(data);
 			line = LINE_3;
 			for(i = 0; i < data->gpu_count * GPUFIELDS; i += GPUFIELDS)
 			{
@@ -355,21 +391,18 @@ static void nrefresh(NThrd *refr)
 				line++;
 				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1CORECLOCK + i], data->tab_graphics[VALUE][GPU1CORECLOCK     + i]);
 				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1MEMCLOCK  + i], data->tab_graphics[VALUE][GPU1MEMCLOCK      + i]);
-				line += GPUFIELDS - 1;
+				line += LINE_3 - 1;
 			}
 			break;
 		case NO_BENCH:
-			for(i = LINE_1; i < LINE_3; i++)
-				wclrline(win, i, info.tb, info.te);
+			do_refresh(data);
 			mvwprintw2c(win, LINE_1, info.tb, "%13s: %s", data->tab_bench[NAME][PRIMESLOWSCORE], data->tab_bench[VALUE][PRIMESLOWSCORE]);
 			mvwprintw2c(win, LINE_2, info.tb, "%13s: %s", data->tab_bench[NAME][PRIMESLOWRUN],   data->tab_bench[VALUE][PRIMESLOWRUN]);
-			for(i = LINE_5; i < LINE_7; i++)
-				wclrline(win, i, info.tb, info.te);
 			mvwprintw2c(win, LINE_5, info.tb, "%13s: %s", data->tab_bench[NAME][PRIMEFASTSCORE], data->tab_bench[VALUE][PRIMEFASTSCORE]);
 			mvwprintw2c(win, LINE_6, info.tb, "%13s: %s", data->tab_bench[NAME][PRIMEFASTRUN],   data->tab_bench[VALUE][PRIMEFASTRUN]);
 			break;
 		default:
-			break;
+			return;
 	}
 
 	wrefresh(win);
@@ -378,35 +411,35 @@ static void nrefresh(NThrd *refr)
 /* Print how to use this TUI */
 static void print_help()
 {
+	timeout(-1);
 	nodelay(stdscr, FALSE);
-	timeout(99999);
 
 	printw(_("Welcome in %s NCurses help!\n"), PRGNAME);
-	printw(_("This help describes how to use this Text-based User Interface.\n"));
+	printw("%s", _("This help describes how to use this Text-based User Interface.\n"));
 
-	printw(_("\nGlobal keys:\n"));
-	printw(_("\tPress 'left' key to switch in left tab.\n"));
-	printw(_("\tPress 'right' key to switch in right tab.\n"));
-	printw(_("\tPress 'h' key to see this help.\n"));
-	printw(_("\tPress 'q' key to exit.\n"));
+	printw("%s", _("\nGlobal keys:\n"));
+	printw("%s", _("\tPress 'left' key to switch in left tab.\n"));
+	printw("%s", _("\tPress 'right' key to switch in right tab.\n"));
+	printw("%s", _("\tPress 'h' key to see this help.\n"));
+	printw("%s", _("\tPress 'q' key to exit.\n"));
 
-	printw(_("\nCPU tab:\n"));
-	printw(_("\tPress 'down' key to decrease core number to monitor.\n"));
-	printw(_("\tPress 'up' key to increase core number to monitor.\n"));
+	printw("%s", _("\nCPU tab:\n"));
+	printw("%s", _("\tPress 'down' key to decrease core number to monitor.\n"));
+	printw("%s", _("\tPress 'up' key to increase core number to monitor.\n"));
 
-	printw(_("\nCaches tab:\n"));
-	printw(_("\tPress 'down' key to switch to previous test.\n"));
-	printw(_("\tPress 'up' key' to switch to next test.\n"));
+	printw("%s", _("\nCaches tab:\n"));
+	printw("%s", _("\tPress 'down' key to switch to previous test.\n"));
+	printw("%s", _("\tPress 'up' key' to switch to next test.\n"));
 
-	printw(_("\nBench tab:\n"));
-	printw(_("\tPress 'down' key to decrement benchmark duration.\n"));
-	printw(_("\tPress 'up' key to increment benchmark duration.\n"));
-	printw(_("\tPress 'next page' key to decrement number of threads to use.\n"));
-	printw(_("\tPress 'previous page' key to increment number of threads to use.\n"));
-	printw(_("\tPress 's' key to start/stop prime numbers (slow) benchmark.\n"));
-	printw(_("\tPress 'f' key to start/stop prime numbers (fast) benchmark.\n"));
+	printw("%s", _("\nBench tab:\n"));
+	printw("%s", _("\tPress 'down' key to decrement benchmark duration.\n"));
+	printw("%s", _("\tPress 'up' key to increment benchmark duration.\n"));
+	printw("%s", _("\tPress 'next page' key to decrement number of threads to use.\n"));
+	printw("%s", _("\tPress 'previous page' key to increment number of threads to use.\n"));
+	printw("%s", _("\tPress 's' key to start/stop prime numbers (slow) benchmark.\n"));
+	printw("%s", _("\tPress 'f' key to start/stop prime numbers (fast) benchmark.\n"));
 
-	printw(_("\nPress any key to exit this help.\n"));
+	printw("%s", _("\nPress any key to exit this help.\n"));
 
 	refresh();
 	getch();
@@ -414,41 +447,6 @@ static void print_help()
 	timeout(opts->refr_time * 1000);
 }
 
-/* Ask for update when a new version is available (portable version only) */
-static void print_new_version()
-{
-	nodelay(stdscr, FALSE);
-
-	printw(_("A new version of %s is available!"), PRGNAME);
-	printw("\n\n");
-#if 0
-	printw(_("Do you want to update %s to version %s after exit?\n"
-		"It will erase this binary file (%s) by the new version."),
-		PRGNAME, new_version[0], binary_name);
-	printw("\n");
-	printw(_("If you want to update, press 'u' key, or anything else to skip.\n"), PRGNAME);
-	refresh();
-
-	if(getch() == 'u')
-	{
-		opts->update = true;
-		printw(_("\n\n%s will be updated."), PRGNAME);
-		refresh();
-		napms(1000);
-	}
-#else
-	printw(_("Version %s is available. Unfortunately, this portable format is now deprecated in favor of AppImage.\n"
-		"You can download %s AppImage on the official download page.\n"),
-		new_version[0], PRGNAME);
-	printw("\n");
-	printw(_("Download page: %s\n"), DOWNLOAD);
-	refresh();
-	getch();
-#endif
-
-	nodelay(stdscr, TRUE);
-}
-
 /* The main window (title, tabs, footer) */
 static void main_win(WINDOW *win, const SizeInfo info, Labels *data)
 {
@@ -460,7 +458,7 @@ static void main_win(WINDOW *win, const SizeInfo info, Labels *data)
 		wattrset(win, COLOR_PAIR(DEFAULT_COLOR));
 	}
 	wclrscr(win);
-	box(win, 0 , 0);
+	box(win, 0, 0);
 
 	mvwprintwc(win, TITLE_LINE, info.width / 2 - strlen(PRGNAME) / 2, TITLE_COLOR, PRGNAME);
 	mvwprintwc(win, HEADER_LINE, 2, DEFAULT_COLOR, PRGNAME);
@@ -484,12 +482,14 @@ static void main_win(WINDOW *win, const SizeInfo info, Labels *data)
 }
 
 /* Display active Core in CPU tab */
+#define BUFF_SIZE 16
 static void print_activecore(WINDOW *win)
 {
-	char buff[16];
-	sprintf(buff, _("Core #%i"), opts->selected_core);
-	mvwprintwc(win, LINE_17, 4, DEFAULT_COLOR, buff);
+	char buff[BUFF_SIZE];
+	snprintf(buff, BUFF_SIZE, _("Core #%i"), opts->selected_core);
+	mvwprintwc(win, LINE_18, 4, DEFAULT_COLOR, buff);
 }
+#undef BUFF_SIZE
 
 /* CPU tab */
 static void ntab_cpu(WINDOW *win, const SizeInfo info, Labels *data)
@@ -561,16 +561,14 @@ static void ntab_cpu(WINDOW *win, const SizeInfo info, Labels *data)
 /* Display active Test in Caches tab */
 static void print_activetest(WINDOW *win, const SizeInfo info, Labels *data)
 {
+	UNUSED(info);
 	const int line = LINE_1 + (CACHEFIELDS + 2) * data->cache_count;
 
 	if(!data->cache_count)
 		return;
 
 	if(HAS_BANDWIDTH)
-	{
-		wclrline(win, line, info.tb, info.width - 2);
 		mvwprintwc(win, line, 12, DEFAULT_COLOR, "%s", data->w_data->test_name[opts->bw_test]);
-	}
 
 	wrefresh(win);
 }
@@ -701,9 +699,6 @@ static void draw_bar(WINDOW *win, const SizeInfo info, Labels *data, int bar)
 	mvwprintwc(win, line, start, DEFAULT_COLOR, "[");
 	mvwprintwc(win, line, end,   DEFAULT_COLOR, "]");
 
-	/* Clean existing bar */
-		wclrline(win, line, start + 1, end);
-
 	/* Draw bar */
 	for(i = 0; i < bar_count; i++)
 	{
@@ -739,6 +734,10 @@ static void ntab_graphics(WINDOW *win, const SizeInfo info, Labels *data)
 
 		switch(i)
 		{
+			case GPU1DRIVER:
+			case GPU2DRIVER:
+			case GPU3DRIVER:
+			case GPU4DRIVER:
 			case GPU1USAGE:
 			case GPU2USAGE:
 			case GPU3USAGE:
@@ -760,7 +759,6 @@ static void ntab_graphics(WINDOW *win, const SizeInfo info, Labels *data)
 /* Display Duration parameter in Bench tab */
 static void print_paramduration(WINDOW *win, const SizeInfo info, Labels *data)
 {
-	wclrline(win, LINE_9, info.tb, info.tm);
 	casprintf(&data->tab_bench[VALUE][PARAMDURATION], true, _("%u mins"), data->b_data->duration);
 	mvwprintw2c(win, LINE_9, info.tb, "%13s: %s", data->tab_bench[NAME][PARAMDURATION], data->tab_bench[VALUE][PARAMDURATION]);
 	wrefresh(win);
