@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2019 Xorg
+*    Copyright © 2014-2020 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 
 #include <ncurses.h>
 
+#define DEFAULT_TERM "xterm"
+
 enum EnLines
 {
 	WINDOW_TOP_LINE,
@@ -95,12 +97,12 @@ typedef struct
 
 /********************************** TUI  **********************************/
 
+/* Put window in the center of the screen */
+static bool resize_window(WINDOW *pwin, const SizeInfo info);
+
 /* Clean window */
 static void wclrscr(WINDOW *pwin);
 
-/* Clean line */
-static void wclrline(WINDOW *pwin, enum EnLines line, unsigned start, unsigned end);
-
 /* Similar to mvwprintw, but specify a color pair */
 static int mvwprintwc(WINDOW *win, int y, int x, enum EnColors pair, const char *fmt, ...);
 
@@ -113,9 +115,6 @@ static void nrefresh(NThrd *refr);
 /* Print how to use this TUI */
 static void print_help(void);
 
-/* Ask for update when a new version is available (portable version only) */
-static void print_new_version(void);
-
 /* The main window (title, tabs, footer) */
 static void main_win(WINDOW *win, const SizeInfo info, Labels *data);
 
